﻿# -------------------------------------------------------------------------------------------------
#  <copyright file="AzureMigrateInstaller.ps1" company="Microsoft">
#      Copyright (c) Microsoft Corporation. All rights reserved.
#  </copyright>
#
#  Description: This script prepares the host machine for various Azure Migrate Scenarios.

#  Version: 2.0.0.0

#  Requirements: 
#       Refer Readme.html for machine requirements 
#       Following files should be placed in the same folder as this script before execution:
#            Scripts : WebBinding.ps1 and SetRegistryForTrustedSites.ps1
#            MSIs    : Microsoft Azure Hyper-V\Server\VMware Assessment Service.msi
#                      Microsoft Azure Hyper-V\Server\VMware Discovery Service.msi
#                      MicrosoftAzureApplianceConfigurationManager.msi
#                      MicrosoftAzureAutoUpdate.msi
#                      MicrosoftAzureDraService.msi     (VMware Migration only)
#                      MicrosoftAzureGatewayService.exe (VMware Migration only)
#            Config  : Scenario.json {"Scenario" : "HyperV|Physical|VMware"}
# -------------------------------------------------------------------------------------------------

#Requires -RunAsAdministrator


param(
    [Parameter(Mandatory=$false)] 
    [ValidateNotNullOrEmpty()] 
    [ValidateSet('HyperV','Physical','VMware')]
    [string]
    $Scenario,

    [Parameter(Mandatory=$false)] 
    [switch]
    $SkipSettingTrustedHostList
)

## This routine writes the output string to the console and also to a log file.
function Log-Info([string] $OutputText)
{
    Write-Host $OutputText -ForegroundColor White
    $OutputText = [string][DateTime]::Now + " " + $OutputText
    $OutputText | %{ Out-File -filepath $InstallerLog -inputobject $_ -append -encoding "ASCII" }
}

function Log-Success([string] $OutputText)
{
    Write-Host $OutputText -ForegroundColor Green
    $OutputText = [string][DateTime]::Now + " " + $OutputText
    $OutputText | %{ Out-File -filepath $InstallerLog -inputobject $_ -append -encoding "ASCII" }
}

## This routine writes the output string to the console and also to a log file.
function Log-Warning([string] $OutputText)
{
    Write-Host $OutputText -ForegroundColor Yellow
    $OutputText = [string][DateTime]::Now + " " + $OutputText
    $OutputText | %{ Out-File -filepath $InstallerLog -inputobject $_ -append -encoding "ASCII"  }
}

## This routine writes the output string to the console and also to a log file.
function Log-Error([string] $OutputText)
{
    Write-Host $OutputText -ForegroundColor Red
    $OutputText = [string][DateTime]::Now + " " + $OutputText
    $OutputText | %{ Out-File -filepath $InstallerLog -inputobject $_ -append -encoding "ASCII" }
}

## Global Initialization
$global:WarningCount      = 0
$global:ReuseScenario     = 0
$global:SelectedScenario  = "Unknown"

$TimeStamp                = [DateTime]::Now.ToString("yyyy-MM-dd-HH-mm-ss")
$LogFileDir               = "$env:ProgramData`\Microsoft Azure\Logs"
$ConfigFileDir            = "$env:ProgramData`\Microsoft Azure\Config"
# TODO: Move reading this path from registry if it exists
$CredFileDir              = "$env:ProgramData`\Microsoft Azure\CredStore"
$ApplianceVersionFilePath = "$Env:SystemDrive`\Users\Public\Desktop\ApplianceVersion.txt"

$HyperVAssessmentServiceMSI    = "Microsoft Azure Hyper-V Assessment Service.msi"
$HyperVDiscoveryServiceMSI     = "Microsoft Azure Hyper-V Discovery Service.msi"
$ServerAssessmentServiceMSI    = "Microsoft Azure Server Assessment Service.msi"
$ServerDiscoveryServiceMSI     = "Microsoft Azure Server Discovery Service.msi"
$VMWareAssessmentServiceMSI    = "Microsoft Azure VMware Assessment Service.msi"
$VMWareDiscoveryServiceMSI     = "Microsoft Azure VMware Discovery Service.msi"

$AssessmentServiceMSILog = "$LogFileDir\AssessmentInstaller_$TimeStamp.log"
$DiscoveryServiceMSILog  = "$LogFileDir\DiscoveryInstaller_$TimeStamp.log"

$GatewayExeName          = "MicrosoftAzureGatewayService.exe"
$DraMsiName              = "MicrosoftAzureDRAService.msi"
$DraMsiLog               = "$LogFileDir\DRAInstaller_$TimeStamp.log"

$WebAppMSI               = "MicrosoftAzureApplianceConfigurationManager.msi"
$WebAppMSILog            = "$LogFileDir\ConfigurationManagerInstaller_$TimeStamp.log"
$ApplianceJsonFilePath   = "$ConfigFileDir\appliance.json"
$ApplianceJsonFileData   = @{
    "Cloud"="Public";
    "ComponentVersion"="1.1.0.1";
    "FabricType"="$global:SelectedScenario";
    "VddkInstallerFolder"="";
    "IsApplianceRegistered"="false";
    "EnableProxyBypassList"="false";
    "ProviderId"="8416fccd-8af8-466e-8021-79db15038c87";
    "AppInsightsInstrumentationKey"="90b46585-1513-4a55-814c-bd57e9b24709"
}

$AutoUpdaterMSI          = "MicrosoftAzureAutoUpdate.msi"
$AutoUpdaterMSILog       = "$LogFileDir\AutoUpdateInstaller_$TimeStamp.log"
$AutoUpdaterJsonFilePath = "$ConfigFileDir\AutoUpdater.json"
$AutoUpdaterJsonFileData = @{
    "ComponentVersion"="1.1.0.1";
    "AppInsightsInstrumentationKey"="90b46585-1513-4a55-814c-bd57e9b24709";
    "AutoUpdateEnabled"="True";
    "ProviderId"="8416fccd-8af8-466e-8021-79db15038c87";
    "AutoUpdaterDownloadLink"="https://aka.ms/latestapplianceservices"
}

$RegAzureAppliancePath = "HKLM:\SOFTWARE\Microsoft\Azure Appliance"
$RegAzureCredStorePath = "HKLM:\Software\Microsoft\AzureAppliance"

## Creating the logfile
New-Item -ItemType Directory -Force -Path $LogFileDir | Out-Null
$InstallerLog = "$LogFileDir\AzureMigrateScenarioInstaller_$TimeStamp.log"
Log-Success "Log file created `"$InstallerLog`" for troubleshooting purpose.`n"

<#
.SYNOPSIS
Create JsonFile
Usage:
    DetectAndCleanupPreviousInstallation
#>
function DetectAndCleanupPreviousInstallation
{ 
    if([System.IO.File]::Exists($ApplianceJsonFilePath))
    {
        do
        { 
            Log-Warning "This host has already been used as Azure Migrate Appliance. If you choose to proceed with this fresh installation, configuration files from the previous installation will be lost. `nDo you still want to continue, [Y/N]"            
            $userChoice = Read-Host

        }while (“y”, ”Y”, "N", "n" -NotContains $userChoice) 

        if ($userChoice -eq "n")
        {
            Log-Warning "Aborting installation..."
            exit 0
        }
        else
        {
            $global:ReuseScenario = 1

            UnInstallProgram("Microsoft Azure Server Assessment Service")
            UnInstallProgram("Microsoft Azure Server Discovery Service")
            UnInstallProgram("Microsoft Azure Hyper-V Assessment Service")
            UnInstallProgram("Microsoft Azure Hyper-V Discovery Service")
            UnInstallProgram("Microsoft Azure VMware Assessment Service")
            UnInstallProgram("Microsoft Azure VMware Discovery Service")
            UnInstallProgram("Microsoft Azure Appliance Auto Update")
            UnInstallProgram("Microsoft Azure Appliance Configuration Manager")
            UnInstallProgram("Microsoft Azure Dra Service")
            UnInstallProgram("Microsoft Azure Gateway Service")

            Log-info "Cleaning up previous configuration files and settings..."

            #Restart IIS
            iisreset.exe /restart | Out-Null

            if([System.IO.File]::Exists($ApplianceVersionFilePath))
            {
                Remove-Item -path $ApplianceVersionFilePath -Force
            }

            if([System.IO.File]::Exists($AutoUpdaterJsonFilePath))
            {
                Remove-Item –path $AutoUpdaterJsonFilePath -Force
            }

            if(Test-Path $RegAzureAppliancePath)
            {
                Remove-Item $RegAzureAppliancePath -Force
            }

            if (Test-Path $RegAzureCredStorePath)
            {
                Remove-Item -Path $RegAzureCredStorePath -Force
            }

            if (Test-Path $ConfigFileDir -PathType Any)
            {
                Remove-Item –path $ApplianceJsonFilePath
                Remove-Item -Recurse -Force $ConfigFileDir
            }

            if (Test-Path $CredFileDir -PathType Container)
            {
                Remove-Item -Recurse -Force $CredFileDir
            }

            if (Test-Path $LogFileDir -PathType Container)
            {
                # Remove all folders under Log folder.
                Get-ChildItem -Recurse $LogFileDir | Where { $_.PSIsContainer } | Remove-Item -Recurse -Force
            }

            if ($Error.Count -eq 0)
            {
                Log-Success "[OK]`n"
            }
            else
            {
                Log-Error "Failure in cleanup. Aborting..."
                Log-Warning "Please take remedial action on the below error or contact Microsoft Support for help."
                Log-Error $Error
                exit -2
            }
        }
    }
}

<#
.SYNOPSIS
Install MSI
Usage:
    UnInstallProgram -ProgramCaption $ProgramCaption
#>

function UnInstallProgram
{
    param(
        [string] $ProgramCaption
        )

    $app = Get-WmiObject -Class Win32_Product -Filter "Caption = '$ProgramCaption' "

    if ($app)
    {
        Log-Warning "$ProgramCaption found installed. Proceeding with uninstallation."
        $app.Uninstall()   
        
        if ($?)
        {
            Log-Success "[Uninstall Successful]`n"
        }
        else
        {
            $global:WarningCount++
            Log-Warning "Warning #$global:WarningCount : Unable to uninstall successfully. Please manually uninstall $ProgramCaption from Control Panel. Continuing..."
        } 
    }

    $Error.Clear()
}

<#
.SYNOPSIS
Install MSI
Usage:
    InstallMSI -MSIFilePath $MSIFilePath -MSIInstallLogName $MSIInstallLogName
#>

function InstallMSI
{ 
    param(
        [string] $MSIFilePath,
        [string] $MSIInstallLogName
        )

    Log-Info "Installing $MSIFilePath..."

    if (-Not (Test-Path -Path $MSIFilePath -PathType Any))
    {
        Log-Error "MSI not found: $MSIFilePath. Aborting..."
        Log-Warning "Please ensure all MSIs are copied to the same folder as the current script."
        exit -3
    }

    $process = (Start-Process -Wait -Passthru -FilePath msiexec -ArgumentList `
        "/i `"$MSIFilePath`" /quiet /lv `"$MSIInstallLogName`"")

    $returnCode = $process.ExitCode;
    
    if ($returnCode -eq 0 -or $returnCode -eq 3010) 
    {
        Log-Success "[OK]`n"
    }
    else
    {
        Log-Error "$MSIFilePath installation failed. More logs available at $MSIInstallLogName. Aborting..."
        Log-Warning "Please refer to http://www.msierrors.com/ to get details about the error code: $returnCode. Please share the installation log file $MSIInstallLogName while contacting Microsoft Support."
        exit -3
    }
}

<#
.SYNOPSIS
Create JsonFile
Usage:
    CreateJsonFile -JsonFileData $JsonFileData -JsonFilePath $JsonFilePath
#>
function CreateJsonFile
{ 
    param(
        $JsonFileData,
        [string] $JsonFilePath
        )

    Log-Info "Creating config File at $JsonFilePath..."

    New-Item -Path $ConfigFileDir -type directory -Force | Out-Null
    $JsonFileData | ConvertTo-Json | Add-Content -Path $JsonFilePath -Encoding UTF8

    if ($?)
    {
        Log-Success "[OK]`n"
    }
    else
    {
        Log-Error "Failure in creating $JsonFilePath. Aborting..."
        exit -4
    }
}

<#
.SYNOPSIS
Enables IIS modules.
Usage:
    EnableIIS 
#>

function EnableIIS
{
    Log-Info "Enabling IIS Role and dependent features..."

    Install-WindowsFeature PowerShell-ISE, WAS, WAS-Process-Model, WAS-Config-APIs, Web-Server, `
        Web-WebServer, Web-Mgmt-Service, Web-Request-Monitor, Web-Common-Http, Web-Static-Content, `
        Web-Default-Doc, Web-Dir-Browsing, Web-Http-Errors, Web-App-Dev, Web-CGI, Web-Health, `
        Web-Http-Logging, Web-Log-Libraries, Web-Security, Web-Filtering, Web-Performance, `
        Web-Stat-Compression, Web-Mgmt-Tools, Web-Mgmt-Console, Web-Scripting-Tools, `
        Web-Asp-Net45, Web-Net-Ext45, Web-Http-Redirect, Web-Windows-Auth, Web-Url-Auth 

    if ($?)
    {
        Log-Success "[OK]`n"
    }
    else
    {
        Log-Error "Failure to enable the required role(s) with error $Errors. Aborting..."
        Log-Warning "Please ensure the following roles are enabled manually: PowerShell-ISE, `
            WAS (Windows Activation Service), WAS-Process-Model, WAS-Config-APIs, Web-Server (IIS), '
            Web-WebServer, Web-Mgmt-Service, Web-Request-Monitor, Web-Common-Http, Web-Static-Content, '
            Web-Default-Doc, Web-Dir-Browsing, Web-Http-Errors, Web-App-Dev, Web-CGI, Web-Health,'
            Web-Http-Logging, Web-Log-Libraries, Web-Security, Web-Filtering, Web-Performance, '
            Web-Stat-Compression, Web-Mgmt-Tools, Web-Mgmt-Console, Web-Scripting-Tools, '
            Web-Asp-Net45, Web-Net-Ext45, Web-Http-Redirect, Web-Windows-Auth, Web-Url-Auth"
        exit -5
    }
}

<#
.SYNOPSIS
Add AzureCloud registry which used to identify NationalCloud
Usage:
    AddAzureCloudRegistry 
#>

function AddingRegistryKeys
{ 
    Log-Info "Adding\Updating Registry Keys...`n"
    $AzureCloudName = "Public"

    if ( -not (Test-Path $RegAzureAppliancePath))
    {
        Log-Info "`tCreating Registry Node: $RegAzureAppliancePath"
        New-Item -Path $RegAzureAppliancePath -Force | Out-Null
    }
            
    New-ItemProperty -Path $RegAzureAppliancePath -Name AzureCloud -Value $AzureCloudName `
        -Force | Out-Null
    New-ItemProperty -Path $RegAzureAppliancePath -Name Type -Value Physical -Force | Out-Null

    if ( -not (Test-Path $RegAzureCredStorePath))
    {
        Log-Info "`tCreating Registry Node: $RegAzureCredStorePath"
        New-Item -Path $RegAzureCredStorePath -Force | Out-Null
    }

    New-ItemProperty -Path $RegAzureCredStorePath -Name CredStoreDefaultPath `
        -value "%Programdata%\Microsoft Azure\CredStore\Credentials.json" -Force | Out-Null

    if ( $?)
    {
        Log-Success "`n[OK]`n"
    }
    else 
    {
        Log-Error "Failed to add\update registry keys. Aborting..."
        Log-Warning "Please ensure that the current user has access to adding registry keys under the path: $RegAzureAppliancePath or $RegAzureCredStorePath"
        exit -6
    }
}

<#
.SYNOPSIS
Validate OS version
Usage:
    ValidateOSVersion
#>
function ValidateOSVersion
{
    [System.Version]$ver = "0.0"
    [System.Version]$minVer = "10.0"

    Log-Info "Verifying supported Operating System version..."

    $OS = Get-WmiObject Win32_OperatingSystem
    $ver = $OS.Version

    If ($ver -lt $minVer)
    {
        Log-Error "The os version is $ver, minimum supported version is $minVer. Aborting..."
        exit -7
    }
    elseif ($OS.Caption.contains("Server") -eq $false)
    {
        Log-Error "OS should be Windows Server 2016. Aborting..."
        exit -8
    }
    else
    {
        Log-Success "[OK]`n"
    }
}


<#
.SYNOPSIS
Run powershell script to set registry for trusted sites. 
Usage:
    ExecutePSScripts -ScriptFilePath $ScriptFilePath
#>

function ExecutePSScripts
{ 
    param(
        [string] $ScriptFilePath
        )

    Log-Info "Running powershell script $ScriptFilePath..."
    if (-Not (Test-Path -Path $ScriptFilePath -PathType Any))
    {
        Log-Error "Script file not found: $ScriptFilePath. Aborting..."
        Log-Warning "Please download the package again and retry."
        exit -9
    }

    & "$ScriptFilePath" | Out-Null

    if ($?)
    {
        Log-Success "[OK]`n"
    }
    else 
    {
        Log-Error "Failed to execute script $ScriptFilePath with error $Error. Aborting..."
        exit -9
    }
}

<#
.SYNOPSIS
custom script run after the Windows Setup process.
Usage:
    CreateApplianceVersionFile
#>

function CreateApplianceVersionFile
{
    Log-Info "Creating Appliance Version File..."
    $ApplianceVersion = "3." + (Get-Date).ToString('yy.MM.dd')
    $fileContent = "$ApplianceVersion"
    
    # Create Appliance version text file.
    New-Item $ApplianceVersionFilePath -ItemType File -Value $ApplianceVersion -Force | Out-Null
    Set-ItemProperty $ApplianceVersionFilePath -name IsReadOnly -value $true

    if ($?)
    {
        Log-Success "[OK]`n"
    }
    else 
    {
        Log-Warning "Failed to create Appliance Version file with at $ApplianceVersionFilePath. Continuing..."
    }
}

<#
.SYNOPSIS
Download and install IIS rewrite module.
Usage:
    InstallRewriteModule
#>

function InstallRewriteModule
{
    # Add check to validate if the module is already installed and skip download and install again.
    Log-Info "Downloading and Installing the IIS URL Rewrite Module 2..."

    $rewriteFile = "$env:SystemDrive\rewrite_amd64.msi"
    $iisrewriteurl = 
        "https://download.microsoft.com/download/C/9/E/C9E8180D-4E51-40A6-A9BF-776990D8BCA9/rewrite_amd64.msi"
    
    Invoke-WebRequest -Uri $iisrewriteurl -OutFile $rewriteFile
    
    $process = (Start-Process -Wait -Passthru -FilePath "MsiExec.exe" `
        -ArgumentList "/i $rewriteFile /qn /norestart")

    Start-Sleep -Seconds 20
    Remove-Item –path $rewriteFile

    $returnCode = $process.ExitCode;

    if ($returnCode -eq 0 -or $returnCode -eq 3010)
    {
        Log-Success "[OK]`n"
    }
    else
    {
        Log-Error "IIS URL Rewrite Module 2 installation failed. Aborting..."
        exit -10
    }
}

<#
.SYNOPSIS
Validate and exit if minimum defined PowerShell version is not available.
Usage:
    ValidatePSVersion
#>

function ValidatePSVersion
{
    [System.Version]$minVer = "4.0"

    Log-Info "Verifying the PowerShell version to run the script..."

    if ($PSVersionTable.PSVersion)
    {
        $global:PsVer = $PSVersionTable.PSVersion
    }
    
    If ($global:PsVer -lt $minVer)
    {
        Log-Error "PowerShell version $minVer, or higher is required. Current PowerShell version is $global:PsVer. Aborting..."
        exit -11;
    }
    else
    {
        Log-Success "[OK]`n"
    }
}

<#
.SYNOPSIS
Ensure IIS backend services are in running state. During IISReset they can remain in stop state as well.
Usage:
    StartIISServices
#>

function StartIISServices
{
    Log-Info "Ensuring critical services for Configuration Manager are runing..."

    Start-service -Name WAS
    Start-service -Name W3SVC

    if ($?)
    {
        Log-Success "[OK]`n"
    } else 
    {
        Log-Error "Failed to start services WAS/W3SVC. Aborting..."
        Log-Warning "Manually start the services WAS and W3SVC"
        exit -12
    }
}

<#
.SYNOPSIS
Set Trusted Hosts in the host\current machine.
Usage:
    SetTrustedHosts
#>

function SetTrustedHosts
{
    $currentList = Get-Item WSMan:\localhost\Client\TrustedHosts
    Log-Info "The current value of $($currentList.Name) = $($currentList.Value)"

    if ($SkipSettingTrustedHostList)
    {
        $global:WarningCount++
        Log-Warning "Warning #$global:WarningCount : Skipping setting Trusted Host List for WinRM. Please manually set Trusted Host list to Windows hosts/servers that will be accessed from this appliance/machine."
        Log-Warning "Not specifying workgroup hosts in Trusted Host list leads to Validate-Operation failure during onboarding through Azure Migrate Appliance Configuration Manager. Continuing..."

        return
    }

    if($currentList -ne $null)
    {
        # Need to add a better than *. * will be used for preview.
        $list = "*"
        Log-Info "Adding $($list) as trusted hosts to the current host machine..."

        Set-Item WSMan:\localhost\Client\TrustedHosts $list.ToString() -Force

        if ($?)
        {
            Log-Success "[OK]`n"
        }
        else
        {
            Log-Error "Failure in adding trusted hosts. Aborting..."
            Log-Warning "Please use -SkipSettingTrustedHostList flag to skip this step and rerun this script."
            exit -13
        }
    }
    else
    {
            Log-Error "Unable to get trusted host list. Aborting..."
            exit -13
    }
}

<#
.SYNOPSIS
Detect which scenario needs to be deployed.
Usage:
    DetectScenario
#>

function DetectScenario
{
    $scenarioFilePath = "$PSScriptRoot\Scenario.json"
    $expectedScenarioList = "HyperV","Physical","VMware"

    if ($Scenario)
    {
        Log-Warning "Scenario to be installed selected through overide parameter: $Scenario`n"
        $global:SelectedScenario = $Scenario
        return
    }

    if (Test-path -PathType Any -Path $ScenarioFilePath)
    {
        $jsonContent = Get-Content $ScenarioFilePath | Out-String | ConvertFrom-Json
        if($expectedScenarioList -contains $jsonContent.Scenario)
        {
            $global:SelectedScenario = $jsonContent.Scenario
            Log-Success "Scenario to be installed: $global:SelectedScenario`n"
        }
        else
        {
            Log-Error "Unknown scenario '$jsonContent' in $scenarioFilePath. Aborting..."
            Log-Warning "Use -Scenario parameter to execute the script."
            Exit -14
        }        
    }
    else
    {
        Log-Error "Scenario.json not found. Aborting..."
        Log-Warning "Use -Scenario parameter to execute the script."
        exit -15
    }
}

<#
.SYNOPSIS
Install Gateway service.
Usage:
    InstallGatewayService -$gatewayPackagerPath "$$gatewayPackagerPath" -$MSIInstallLogName "ToDo"
#>

function InstallGatewayService
{ 
    param(
        [string] $gatewayPackagerPath,
        [string] $MSIInstallLogName
    )

    $extractCmd = "`"$gatewayPackagerPath`"" + " /q /x:`"$PSScriptRoot`""
    
    Log-Info "Extracting and Installing Gateway Service..."
    
    Invoke-Expression "& $extractCmd"
    Start-Sleep -Seconds 5

    $process = (Start-Process -Wait -Passthru -FilePath "$PSScriptRoot\GATEWAYSETUPINSTALLER.EXE" -ArgumentList "CommandLineInstall ")
    $returnCode = $process.ExitCode;

    if ($returnCode -eq 0 -or $returnCode -eq 3010) 
    {
        Log-Success "[OK]`n"
    }
    else
    {
        Log-Error "Gateway service installation failed. Aborting..."
        Log-Warning "Please refer to http://www.msierrors.com/ to get details about the error code: $returnCode. Please share the installation log file NONAME while contacting Microsoft Support."
        exit -16
    }
}

try
{
    $Error.Clear()

    # Validate PowerSHell, OS version and user role.
    ValidatePSVersion
    ValidateOSVersion

    # Detect the scenario that needs to be installed.
    DetectScenario

    # Detect and take user intent to cleanup previous installation if found.
    DetectAndCleanupPreviousInstallation
    
    # Add the required registry keys.
    AddingRegistryKeys

    # Enable IIS.
    EnableIIS

    # Download and Install rewrite module.
    InstallRewriteModule
    
    # Set trusted hosts to machine.
    SetTrustedHosts
 
    # Install Discovery, Assessment and MIgration agents based on the scenario .
    switch($global:SelectedScenario)
    {
        HyperV
        {
            $ApplianceJsonFileData.FabricType="HyperV"
            InstallMSI -MSIFilePath "$PSScriptRoot\$HyperVDiscoveryServiceMSI" `
                -MSIInstallLogName $DiscoveryServiceMSILog
            InstallMSI -MSIFilePath "$PSScriptRoot\$HyperVAssessmentServiceMSI" `
                -MSIInstallLogName $AssessmentServiceMSILog
        }
        Physical
        {
            $ApplianceJsonFileData.FabricType="Physical"
            InstallMSI -MSIFilePath "$PSScriptRoot\$ServerDiscoveryServiceMSI" `
                -MSIInstallLogName $DiscoveryServiceMSILog
            InstallMSI -MSIFilePath "$PSScriptRoot\$ServerAssessmentServiceMSI" `
                -MSIInstallLogName $AssessmentServiceMSILog
        }
        VMware
        {
            $ApplianceJsonFileData.FabricType="VMwareV2"
            $ApplianceJsonFileData.VddkInstallerFolder="%programfiles%\\VMware\\VMware Virtual Disk Development Kit";

            InstallMSI -MSIFilePath "$PSScriptRoot\$VMwareDiscoveryServiceMSI" `
                -MSIInstallLogName $DiscoveryServiceMSILog
            InstallMSI -MSIFilePath "$PSScriptRoot\$VMwareAssessmentServiceMSI" `
                -MSIInstallLogName $AssessmentServiceMSILog

            InstallMSI -MSIFilePath "$PSScriptRoot\$DraMsiName" `
                -MSIInstallLogName $DraMsiLog

            # LogFilePath needs to be added.
            InstallGatewayService "$PSScriptRoot\$GatewayExeName" ""
        }
        default
        {
            Log-Error "Unexpected Scenario selected:$.global:SelectedScenario. Aborting..."
            Log-Warning "Please retry the script with -Scenario parameter."
            exit -20
        }
    }

    # Install Appliance Configuration Manager
    CreateJsonFile -JsonFileData $ApplianceJsonFileData -JsonFilePath $ApplianceJsonFilePath
    InstallMSI -MSIFilePath "$PSScriptRoot\$WebAppMSI" -MSIInstallLogName $WebAppMSILog
 
    # Install Agent updater.
    CreateJsonFile -JsonFileData $AutoUpdaterJsonFileData -JsonFilePath $AutoUpdaterJsonFilePath
    InstallMSI -MSIFilePath "$PSScriptRoot\$AutoUpdaterMSI" -MSIInstallLogName $AutoUpdaterMSILog

    # Custom script for IIS bindings and launch UI.
    CreateApplianceVersionFile

    # Ensure critical services for WebApp are in running state.
    StartIISServices

    # Execute powershell scripts
    ExecutePSScripts -ScriptFilePath "$PSScriptRoot\WebBinding.ps1"
    ExecutePSScripts -ScriptFilePath "$PSScriptRoot\SetRegistryForTrustedSites.ps1"

    if ($global:ReuseScenario -eq 1)
    {
        do
        {
            $global:WarningCount++
            Log-Warning "Warning #$global:WarningCount : Please manually clear browser cache before proceeding further.`nProceed [Y] or exit the script now as browser cache cannot be cleared now [N]"            
            $userChoice = Read-Host
        }while (“y”, ”Y”, "N", "n" -NotContains $userChoice) 

        if ($userChoice.ToLower() -eq "n")
        {
            Log-Warning "Installation completed. Use the shortcut on desktop to launch `"Azure Migrate Appliance Configuration Manager`" once the browser cache has been manually cleared"     
        }
        else
        {
            Log-Success "Installation completed successfully. Launching Azure Migrate Configuration Manager to start onboarding process..."
            Log-Success "Use the shortcut on desktop to re-launch `"Azure Migrate Appliance Configuration Manager`""
        }
    }

    if ($global:WarningCount -gt 0)
    {
        Log-Warning "Please review the $global:WarningCount warning(s) hit during script execution and take manual corrective action as suggested in the warning(s)."        
    }
}
catch
{
    Log-Error "Script execution failed with error $_.Exception.Message"
    Log-Error "Error Record: $_.Exception.ErrorRecord"
    Log-Error "Exception caught:  $_.Exception"
    Log-Warning "Retry the script after resolving the issue(s) or contact Microsoft Support."
    exit -1
}
# SIG # Begin signature block
# MIInKwYJKoZIhvcNAQcCoIInHDCCJxgCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAPTEw66P44Tptd
# CCAqUKmMVHSHorYX5gWWlz1kaaZvdKCCEWUwggh3MIIHX6ADAgECAhM2AAAA4kR3
# G3EijXHTAAEAAADiMA0GCSqGSIb3DQEBCwUAMEExEzARBgoJkiaJk/IsZAEZFgNH
# QkwxEzARBgoJkiaJk/IsZAEZFgNBTUUxFTATBgNVBAMTDEFNRSBDUyBDQSAwMTAe
# Fw0xOTAzMjUxMzEzMzRaFw0yMDAzMjQxMzEzMzRaMCQxIjAgBgNVBAMTGU1pY3Jv
# c29mdCBBenVyZSBDb2RlIFNpZ24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEK
# AoIBAQCq3seGPV6z/qGZ5OKvkCZdIhfURhgoq4H4VtLGQ32NUqeRsbIQAa5cjUvu
# hbgtJlj3z2trVo7R9N1Qxega6uWrUZj8Zh7wTvevxb65aLLpr12F/w1LFhy3J7LX
# c0WQQD0uLB9AY2Gln88LuQyC8CvE40lrjm055fxnQmLP4dAXRK0KZA4ml4aHEOQl
# 9hd5Bh0sxn6mHCNMmYR3su+SwebaXWfs/Rluv+lC6Uwu9DgCktYU+O2xhitCupSS
# vSoYwgO5MLW6IfQg5WtA86e+3jFMOCQoZYDk1m6aYi2zuwwzYuk2BdAej5sVvvz5
# ZLDYjvAQy3zndQT10EdkTesjYg0ZAgMBAAGjggWDMIIFfzApBgkrBgEEAYI3FQoE
# HDAaMAwGCisGAQQBgjdbAgEwCgYIKwYBBQUHAwMwPQYJKwYBBAGCNxUHBDAwLgYm
# KwYBBAGCNxUIhpDjDYTVtHiE8Ys+hZvdFs6dEoFghNiuSIWmliYCAWQCAQ0wggJ2
# BggrBgEFBQcBAQSCAmgwggJkMGIGCCsGAQUFBzAChlZodHRwOi8vY3JsLm1pY3Jv
# c29mdC5jb20vcGtpaW5mcmEvQ2VydHMvQlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1F
# JTIwQ1MlMjBDQSUyMDAxKDEpLmNydDBSBggrBgEFBQcwAoZGaHR0cDovL2NybDEu
# YW1lLmdibC9haWEvQlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1FJTIwQ1MlMjBDQSUy
# MDAxKDEpLmNydDBSBggrBgEFBQcwAoZGaHR0cDovL2NybDIuYW1lLmdibC9haWEv
# QlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1FJTIwQ1MlMjBDQSUyMDAxKDEpLmNydDBS
# BggrBgEFBQcwAoZGaHR0cDovL2NybDMuYW1lLmdibC9haWEvQlkyUEtJQ1NDQTAx
# LkFNRS5HQkxfQU1FJTIwQ1MlMjBDQSUyMDAxKDEpLmNydDBSBggrBgEFBQcwAoZG
# aHR0cDovL2NybDQuYW1lLmdibC9haWEvQlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1F
# JTIwQ1MlMjBDQSUyMDAxKDEpLmNydDCBrQYIKwYBBQUHMAKGgaBsZGFwOi8vL0NO
# PUFNRSUyMENTJTIwQ0ElMjAwMSxDTj1BSUEsQ049UHVibGljJTIwS2V5JTIwU2Vy
# dmljZXMsQ049U2VydmljZXMsQ049Q29uZmlndXJhdGlvbixEQz1BTUUsREM9R0JM
# P2NBQ2VydGlmaWNhdGU/YmFzZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0
# aG9yaXR5MB0GA1UdDgQWBBRS47CtDNi0cKTZcPAA30itV+wMEjAOBgNVHQ8BAf8E
# BAMCB4AwUAYDVR0RBEkwR6RFMEMxKTAnBgNVBAsTIE1pY3Jvc29mdCBPcGVyYXRp
# b25zIFB1ZXJ0byBSaWNvMRYwFAYDVQQFEw0yMzYxNjcrNDUzMzgwMIIB1AYDVR0f
# BIIByzCCAccwggHDoIIBv6CCAbuGPGh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9w
# a2lpbmZyYS9DUkwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDEu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDIu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDMu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDQu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIaBumxkYXA6Ly8vQ049
# QU1FJTIwQ1MlMjBDQSUyMDAxLENOPUJZMlBLSUNTQ0EwMSxDTj1DRFAsQ049UHVi
# bGljJTIwS2V5JTIwU2VydmljZXMsQ049U2VydmljZXMsQ049Q29uZmlndXJhdGlv
# bixEQz1BTUUsREM9R0JMP2NlcnRpZmljYXRlUmV2b2NhdGlvbkxpc3Q/YmFzZT9v
# YmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Qb2ludDAfBgNVHSMEGDAWgBQbZqIZ
# /JvrpdqEjxiY6RCkw3uSvTAfBgNVHSUEGDAWBgorBgEEAYI3WwIBBggrBgEFBQcD
# AzANBgkqhkiG9w0BAQsFAAOCAQEAaYwZUZEKV4ZrTtIa0G/BCXxv9n+2jS3Tgnxb
# BlSPriM5WvURcRv83qyIUeswlVbBnbkOsGME5Q/+8w6shIYz03jNeC+3Sq4dO3rC
# vrMalQO35SRX55Xm+LJZS9N/y8/YDs7zuI1zC4g80oBtwfCZ6faeYYX1afCX2LXO
# ekAtLmJzI+n7KFhdo/38JQKF8km4n+A6ELzf+7rwIaJzecHtBmUlZkgWr+rl5g7G
# fbgUUDnNCC0BOKhEdTL6OYCLT1YvmpoqDg+mX4vg52c8ZMS9HVaObacYqKTE6cdR
# pb5fJKKSE0Ybe+PrPp4TwSvfJ2YO/ElKPEMKfZRgYFKRUhpF0jCCCOYwggbOoAMC
# AQICEx8AAAAUtMUfxvKAvnEAAAAAABQwDQYJKoZIhvcNAQELBQAwPDETMBEGCgmS
# JomT8ixkARkWA0dCTDETMBEGCgmSJomT8ixkARkWA0FNRTEQMA4GA1UEAxMHYW1l
# cm9vdDAeFw0xNjA5MTUyMTMzMDNaFw0yMTA5MTUyMTQzMDNaMEExEzARBgoJkiaJ
# k/IsZAEZFgNHQkwxEzARBgoJkiaJk/IsZAEZFgNBTUUxFTATBgNVBAMTDEFNRSBD
# UyBDQSAwMTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBANVXgQLW+frQ
# 9xuAud03zSTcZmH84YlyrSkM0hsbmr+utG00tVRHgw40pxYbJp5W+hpDwnmJgicF
# oGRrPt6FifMmnd//1aD/fW1xvGs80yZk9jxTNcisVF1CYIuyPctwuJZfwE3wcGxh
# kVw/tj3ZHZVacSls3jRD1cGwrcVo1IR6+hHMvUejtt4/tv0UmUoH82HLQ8w1oTX9
# D7xj35Zt9T0pOPqM3Gt9+/zs7tPp2gyoOYv8xR4X0iWZKuXTzxugvMA63YsB4ehu
# SBqzHdkF55rxH47aT6hPhvDHlm7M2lsZcRI0CUAujwcJ/vELeFapXNGpt2d3wcPJ
# M0bpzrPDJ/8CAwEAAaOCBNowggTWMBAGCSsGAQQBgjcVAQQDAgEBMCMGCSsGAQQB
# gjcVAgQWBBSR/DPOQp72k+bifVTXCBi7uNdxZTAdBgNVHQ4EFgQUG2aiGfyb66Xa
# hI8YmOkQpMN7kr0wggEEBgNVHSUEgfwwgfkGBysGAQUCAwUGCCsGAQUFBwMBBggr
# BgEFBQcDAgYKKwYBBAGCNxQCAQYJKwYBBAGCNxUGBgorBgEEAYI3CgMMBgkrBgEE
# AYI3FQYGCCsGAQUFBwMJBggrBgEFBQgCAgYKKwYBBAGCN0ABAQYLKwYBBAGCNwoD
# BAEGCisGAQQBgjcKAwQGCSsGAQQBgjcVBQYKKwYBBAGCNxQCAgYKKwYBBAGCNxQC
# AwYIKwYBBQUHAwMGCisGAQQBgjdbAQEGCisGAQQBgjdbAgEGCisGAQQBgjdbAwEG
# CisGAQQBgjdbBQEGCisGAQQBgjdbBAEGCisGAQQBgjdbBAIwGQYJKwYBBAGCNxQC
# BAweCgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMBIGA1UdEwEB/wQIMAYBAf8CAQAw
# HwYDVR0jBBgwFoAUKV5RXmSuNLnrrJwNp4x1AdEJCygwggFoBgNVHR8EggFfMIIB
# WzCCAVegggFToIIBT4YjaHR0cDovL2NybDEuYW1lLmdibC9jcmwvYW1lcm9vdC5j
# cmyGMWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2lpbmZyYS9jcmwvYW1lcm9v
# dC5jcmyGI2h0dHA6Ly9jcmwyLmFtZS5nYmwvY3JsL2FtZXJvb3QuY3JshiNodHRw
# Oi8vY3JsMy5hbWUuZ2JsL2NybC9hbWVyb290LmNybIaBqmxkYXA6Ly8vQ049YW1l
# cm9vdCxDTj1BTUVST09ULENOPUNEUCxDTj1QdWJsaWMlMjBLZXklMjBTZXJ2aWNl
# cyxDTj1TZXJ2aWNlcyxDTj1Db25maWd1cmF0aW9uLERDPUFNRSxEQz1HQkw/Y2Vy
# dGlmaWNhdGVSZXZvY2F0aW9uTGlzdD9iYXNlP29iamVjdENsYXNzPWNSTERpc3Ry
# aWJ1dGlvblBvaW50MIIBqwYIKwYBBQUHAQEEggGdMIIBmTA3BggrBgEFBQcwAoYr
# aHR0cDovL2NybDEuYW1lLmdibC9haWEvQU1FUk9PVF9hbWVyb290LmNydDBHBggr
# BgEFBQcwAoY7aHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraWluZnJhL2NlcnRz
# L0FNRVJPT1RfYW1lcm9vdC5jcnQwNwYIKwYBBQUHMAKGK2h0dHA6Ly9jcmwyLmFt
# ZS5nYmwvYWlhL0FNRVJPT1RfYW1lcm9vdC5jcnQwNwYIKwYBBQUHMAKGK2h0dHA6
# Ly9jcmwzLmFtZS5nYmwvYWlhL0FNRVJPT1RfYW1lcm9vdC5jcnQwgaIGCCsGAQUF
# BzAChoGVbGRhcDovLy9DTj1hbWVyb290LENOPUFJQSxDTj1QdWJsaWMlMjBLZXkl
# MjBTZXJ2aWNlcyxDTj1TZXJ2aWNlcyxDTj1Db25maWd1cmF0aW9uLERDPUFNRSxE
# Qz1HQkw/Y0FDZXJ0aWZpY2F0ZT9iYXNlP29iamVjdENsYXNzPWNlcnRpZmljYXRp
# b25BdXRob3JpdHkwDQYJKoZIhvcNAQELBQADggIBACi3Soaajx+kAWjNwgDqkIvK
# AOFkHmS1t0DlzZlpu1ANNfA0BGtck6hEG7g+TpUdVrvxdvPQ5lzU3bGTOBkyhGmX
# oSIlWjKC7xCbbuYegk8n1qj3rTcjiakdbBqqHdF8J+fxv83E2EsZ+StzfCnZXA62
# QCMn6t8mhCWBxpwPXif39Ua32yYHqP0QISAnLTjjcH6bAV3IIk7k5pQ/5NA6qIL8
# yYD6vRjpCMl/3cZOyJD81/5+POLNMx0eCClOfFNxtaD0kJmeThwL4B2hAEpHTeRN
# tB8ib+cze3bvkGNPHyPlSHIuqWoC31x2Gk192SfzFDPV1PqFOcuKjC8049SSBtC1
# X7hyvMqAe4dop8k3u25+odhvDcWdNmimdMWvp/yZ6FyjbGlTxtUqE7iLTLF1eaUL
# SEobAap16hY2N2yTJTISKHzHI4rjsEQlvqa2fj6GLxNj/jC+4LNy+uRmfQXShd30
# lt075qTroz0Nt680pXvVhsRSdNnzW2hfQu2xuOLg8zKGVOD/rr0GgeyhODjKgL2G
# Hxctbb9XaVSDf6ocdB//aDYjiabmWd/WYmy7fQ127KuasMh5nSV2orMcAed8CbIV
# I3NYu+sahT1DRm/BGUN2hSpdsPQeO73wYvp1N7DdLaZyz7XsOCx1quCwQ+bojWVQ
# TmKLGegSoUpZNfmP9MtSMYIVHDCCFRgCAQEwWDBBMRMwEQYKCZImiZPyLGQBGRYD
# R0JMMRMwEQYKCZImiZPyLGQBGRYDQU1FMRUwEwYDVQQDEwxBTUUgQ1MgQ0EgMDEC
# EzYAAADiRHcbcSKNcdMAAQAAAOIwDQYJYIZIAWUDBAIBBQCgga4wGQYJKoZIhvcN
# AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUw
# LwYJKoZIhvcNAQkEMSIEIJ62GdrqIPpdCxvJ89/6ZJoh3+78e8DBvfuhuumHywW/
# MEIGCisGAQQBgjcCAQwxNDAyoBSAEgBNAGkAYwByAG8AcwBvAGYAdKEagBhodHRw
# Oi8vd3d3Lm1pY3Jvc29mdC5jb20wDQYJKoZIhvcNAQEBBQAEggEAXtmrTHOB8+FM
# 5pXtEmdr6BqBvV6wV+xq/w5mM17c97hyn2KkBs5boPJ8cYiIoDZha/0GPZKRphVt
# IQRnPAV/Abb7FTAHE3nzeK3iEV5fZxjk4Dk440z9zd7wljRMi8psrmnB0A/WTxyG
# gtJ0XilAqWi6wrO2LZD/Hbi0YCpuJxxwRNGZpsNebDAl9I0DUOTkn/zBx0HmSbEP
# ct9NW/+TzPbYSbtN0UPLwfMvan6T7btJ/D8W4HehgU5NzoaSJfYqls3x8fE3uRPy
# 7VfoSAOlICinqc/NFT8N8jenVe0NTa6Cya0GPJ9H6c96Sdy+t+U9wRzk4nJH8UGo
# 5KwCApa016GCEuQwghLgBgorBgEEAYI3AwMBMYIS0DCCEswGCSqGSIb3DQEHAqCC
# Er0wghK5AgEDMQ8wDQYJYIZIAWUDBAIBBQAwggFQBgsqhkiG9w0BCRABBKCCAT8E
# ggE7MIIBNwIBAQYKKwYBBAGEWQoDATAxMA0GCWCGSAFlAwQCAQUABCDYi0BcVh1o
# CC2JXhCdPHwN8HN00x+c3yp+OkLkULLcKQIGXioHhGk6GBIyMDIwMDEyNjE5MDk0
# Ny4zM1owBIACAfSggdCkgc0wgcoxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNo
# aW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29y
# cG9yYXRpb24xJTAjBgNVBAsTHE1pY3Jvc29mdCBBbWVyaWNhIE9wZXJhdGlvbnMx
# JjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOkFFMkMtRTMyQi0xQUZDMSUwIwYDVQQD
# ExxNaWNyb3NvZnQgVGltZS1TdGFtcCBTZXJ2aWNloIIOPDCCBPEwggPZoAMCAQIC
# EzMAAAEWkyLqv7stTeYAAAAAARYwDQYJKoZIhvcNAQELBQAwfDELMAkGA1UEBhMC
# VVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNV
# BAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRp
# bWUtU3RhbXAgUENBIDIwMTAwHhcNMTkxMTEzMjE0MDM0WhcNMjEwMjExMjE0MDM0
# WjCByjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcT
# B1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjElMCMGA1UE
# CxMcTWljcm9zb2Z0IEFtZXJpY2EgT3BlcmF0aW9uczEmMCQGA1UECxMdVGhhbGVz
# IFRTUyBFU046QUUyQy1FMzJCLTFBRkMxJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1l
# LVN0YW1wIFNlcnZpY2UwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDQ
# +Bvz/b3qJ78uPatar4wiUxnCRJTBmwgp3jva3J5U7I1wqCpIJPjNC/PaR35UPuVh
# Y3dXRRIvmiKl2n7GnvAfsIAcLvM1TbU+N0dY86LaiQiU/JOKStAGjCj4w5X1eB3d
# aoyclIoRqeTtCZIeAmKYnar9lFHn7Rnm5lF6MkeRBwmZMHwar/CDYT+CO5GX+IZr
# h8Ym7RwdJSJfvybkUTJppzVeeSpp9KphHypvuyfdlW3+W1uIZQmvzviiLZtl25S0
# IxHRCJZZ8EwTFwtXgDp6uYL3xxtu+L+lBvWnWXu9dPY1F4P7GQWrMqfRwDdWL1xz
# kuMOFj3UvXxG4ciBTeodAgMBAAGjggEbMIIBFzAdBgNVHQ4EFgQU5b9eK7gX6BfN
# R+UfeTT+V8ghVQswHwYDVR0jBBgwFoAU1WM6XIoxkPNDe3xGG8UzaFqFbVUwVgYD
# VR0fBE8wTTBLoEmgR4ZFaHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwv
# cHJvZHVjdHMvTWljVGltU3RhUENBXzIwMTAtMDctMDEuY3JsMFoGCCsGAQUFBwEB
# BE4wTDBKBggrBgEFBQcwAoY+aHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraS9j
# ZXJ0cy9NaWNUaW1TdGFQQ0FfMjAxMC0wNy0wMS5jcnQwDAYDVR0TAQH/BAIwADAT
# BgNVHSUEDDAKBggrBgEFBQcDCDANBgkqhkiG9w0BAQsFAAOCAQEANoNq19qpo9sD
# L+lJ0rx7xd3M3q/33v01JYtSw0eg0EWnKvUVMZy/B/RIoh97TKC3yzymWQBjExR+
# 1SY6c746cZdSBM6t5cjDjO2x6mi2t4dwvT3j5ufs5QYs8xrnoZCKLdZSCIY8/SB8
# 1eFdypJ130eZeMyEp9GfSM4pEQlM+o8ctdLFwDVJVOuDwkO2QP7JZXO64CRKilMb
# G0mnk9ythSsxvudbVygh7u2xCJI+nrEVXMp9cVQn1XMAKG5wT7ympculX/FTen/W
# 9/QOAtkykjbabACBt78TrCDINIrIdcjhSyIiVJgBK/4iUo61xfOtdXQhFBR1HIgL
# VmJZYv9iCDCCBnEwggRZoAMCAQICCmEJgSoAAAAAAAIwDQYJKoZIhvcNAQELBQAw
# gYgxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdS
# ZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xMjAwBgNVBAMT
# KU1pY3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0eSAyMDEwMB4XDTEw
# MDcwMTIxMzY1NVoXDTI1MDcwMTIxNDY1NVowfDELMAkGA1UEBhMCVVMxEzARBgNV
# BAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jv
# c29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAg
# UENBIDIwMTAwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCpHQ28dxGK
# OiDs/BOX9fp/aZRrdFQQ1aUKAIKF++18aEssX8XD5WHCdrc+Zitb8BVTJwQxH0Eb
# GpUdzgkTjnxhMFmxMEQP8WCIhFRDDNdNuDgIs0Ldk6zWczBXJoKjRQ3Q6vVHgc2/
# JGAyWGBG8lhHhjKEHnRhZ5FfgVSxz5NMksHEpl3RYRNuKMYa+YaAu99h/EbBJx0k
# ZxJyGiGKr0tkiVBisV39dx898Fd1rL2KQk1AUdEPnAY+Z3/1ZsADlkR+79BL/W7l
# msqxqPJ6Kgox8NpOBpG2iAg16HgcsOmZzTznL0S6p/TcZL2kAcEgCZN4zfy8wMlE
# XV4WnAEFTyJNAgMBAAGjggHmMIIB4jAQBgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4E
# FgQU1WM6XIoxkPNDe3xGG8UzaFqFbVUwGQYJKwYBBAGCNxQCBAweCgBTAHUAYgBD
# AEEwCwYDVR0PBAQDAgGGMA8GA1UdEwEB/wQFMAMBAf8wHwYDVR0jBBgwFoAU1fZW
# y4/oolxiaNE9lJBb186aGMQwVgYDVR0fBE8wTTBLoEmgR4ZFaHR0cDovL2NybC5t
# aWNyb3NvZnQuY29tL3BraS9jcmwvcHJvZHVjdHMvTWljUm9vQ2VyQXV0XzIwMTAt
# MDYtMjMuY3JsMFoGCCsGAQUFBwEBBE4wTDBKBggrBgEFBQcwAoY+aHR0cDovL3d3
# dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNSb29DZXJBdXRfMjAxMC0wNi0y
# My5jcnQwgaAGA1UdIAEB/wSBlTCBkjCBjwYJKwYBBAGCNy4DMIGBMD0GCCsGAQUF
# BwIBFjFodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vUEtJL2RvY3MvQ1BTL2RlZmF1
# bHQuaHRtMEAGCCsGAQUFBwICMDQeMiAdAEwAZQBnAGEAbABfAFAAbwBsAGkAYwB5
# AF8AUwB0AGEAdABlAG0AZQBuAHQALiAdMA0GCSqGSIb3DQEBCwUAA4ICAQAH5ohR
# DeLG4Jg/gXEDPZ2joSFvs+umzPUxvs8F4qn++ldtGTCzwsVmyWrf9efweL3HqJ4l
# 4/m87WtUVwgrUYJEEvu5U4zM9GASinbMQEBBm9xcF/9c+V4XNZgkVkt070IQyK+/
# f8Z/8jd9Wj8c8pl5SpFSAK84Dxf1L3mBZdmptWvkx872ynoAb0swRCQiPM/tA6WW
# j1kpvLb9BOFwnzJKJ/1Vry/+tuWOM7tiX5rbV0Dp8c6ZZpCM/2pif93FSguRJuI5
# 7BlKcWOdeyFtw5yjojz6f32WapB4pm3S4Zz5Hfw42JT0xqUKloakvZ4argRCg7i1
# gJsiOCC1JeVk7Pf0v35jWSUPei45V3aicaoGig+JFrphpxHLmtgOR5qAxdDNp9Dv
# fYPw4TtxCd9ddJgiCGHasFAeb73x4QDf5zEHpJM692VHeOj4qEir995yfmFrb3ep
# gcunCaw5u+zGy9iCtHLNHfS4hQEegPsbiSpUObJb2sgNVZl6h3M7COaYLeqN4DMu
# Ein1wC9UJyH3yKxO2ii4sanblrKnQqLJzxlBTeCG+SqaoxFmMNO7dDJL32N79ZmK
# LxvHIa9Zta7cRDyXUHHXodLFVeNp3lfB0d4wwP3M5k37Db9dT+mdHhk4L7zPWAUu
# 7w2gUDXa7wknHNWzfjUeCLraNtvTX4/edIhJEqGCAs4wggI3AgEBMIH4oYHQpIHN
# MIHKMQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
# UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSUwIwYDVQQL
# ExxNaWNyb3NvZnQgQW1lcmljYSBPcGVyYXRpb25zMSYwJAYDVQQLEx1UaGFsZXMg
# VFNTIEVTTjpBRTJDLUUzMkItMUFGQzElMCMGA1UEAxMcTWljcm9zb2Z0IFRpbWUt
# U3RhbXAgU2VydmljZaIjCgEBMAcGBSsOAwIaAxUAh01b3PJPoIsbWoIM1z76zOzd
# lkOggYMwgYCkfjB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQ
# MA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
# MSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EgMjAxMDANBgkqhkiG
# 9w0BAQUFAAIFAOHX0cAwIhgPMjAyMDAxMjYxNjUyMTZaGA8yMDIwMDEyNzE2NTIx
# NlowdzA9BgorBgEEAYRZCgQBMS8wLTAKAgUA4dfRwAIBADAKAgEAAgIcYwIB/zAH
# AgEAAgISBjAKAgUA4dkjQAIBADA2BgorBgEEAYRZCgQCMSgwJjAMBgorBgEEAYRZ
# CgMCoAowCAIBAAIDB6EgoQowCAIBAAIDAYagMA0GCSqGSIb3DQEBBQUAA4GBAETC
# lq/2PLf7gSRt8ov5KDF3hHg4qtDfqzHUdGqiAce7PpraFehdHH3YcKfCxAKOMDRD
# 0tgFFOFwkca5t2M1+gWItIalALeWtbtDGYm94H36vMbs1DAH1V1/jJL4jNifKjy3
# xFRQWubotUuGKHBR8jkZR+KAFYawjGfBWOXxrjPvMYIDDTCCAwkCAQEwgZMwfDEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
# bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWlj
# cm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTACEzMAAAEWkyLqv7stTeYAAAAAARYw
# DQYJYIZIAWUDBAIBBQCgggFKMBoGCSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAv
# BgkqhkiG9w0BCQQxIgQghIV2qovJ5OMt8eWFh62LOb5Xut5m6fFNt4tPKvKafMUw
# gfoGCyqGSIb3DQEJEAIvMYHqMIHnMIHkMIG9BCCDIpT2pGApCJdcKZshNt20sQ2F
# jGogyEFoz6Je0sGQyjCBmDCBgKR+MHwxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpX
# YXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQg
# Q29ycG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1wIFBDQSAy
# MDEwAhMzAAABFpMi6r+7LU3mAAAAAAEWMCIEIDjoDVoLfsvj0jHaTootcNUNr+sn
# 00+iJAzZHRxiuwaTMA0GCSqGSIb3DQEBCwUABIIBAJe8ewACwydm5/ntyouBaxse
# 9QNa4xQ1f20iMjitRlzJYYbHhTCo4UROWxkQadl+DA6CIGNoQEUuoCShH83jT6yY
# KfK6F/FxHB1ATUOrkvNwuSiJ9t9c0wGD4C3sxPWQXVvuQwy1u/kVnkl3TqxGjN3U
# F1ouJ/NrxYDYjGuve6cD6P/q3jNyzzJGrVx1KTIJvCqA2JaSXPQPtP9cGxBX6XuK
# DuGqPCYPQl36/me63p+c6Ufu+chGkIvnGGxc2GAeg/tWQ5Gi8d2YO+Iw5KV1Omzi
# Tcg99OTT0+ZrLmu0zjBRlmZE5/p5xOq1sDs1fvd+ykoJVrLQTwmejOTPknSm6Bs=
# SIG # End signature block
