﻿<#
.Synopsis  
  Creating registry keys for trusted sites.
  
#>

$InternetSettingsRegistryHive = "HKCU:\Software\Microsoft\Windows\CurrentVersion\Internet Settings";
$TrustedSitesEscDomainsRegistryHive = $InternetSettingsRegistryHive + "\ZoneMap\EscDomains";
$TrustedSitesDomainsRegistryHive = $InternetSettingsRegistryHive + "\ZoneMap\Domains";
$InternetExplorerRegistryHive = "HKCU:\Software\Microsoft\Internet Explorer\New Windows";
$regKeyIEAllow = $InternetExplorerRegistryHive + "\Allow";
$regKeyEscBlankDomain = $TrustedSitesEscDomainsRegistryHive + "\blank";
$regKeyBlankDomain = $TrustedSitesDomainsRegistryHive + "\blank";
$AzureMigrateApplianceHive = "HKLM:\Software\Microsoft\Azure Migrate Appliance";
$AzureCloudKeyName = "AzureCloud";
$machineHostName = (Get-WmiObject win32_computersystem).DNSHostName
$LogFileDir = "$env:SystemRoot\Setup"
$logName = $LogFileDir + "\SetRegistry-" + [DateTime]::Now.ToString("yyyy-MM-dd-HH-mm-ss") + ".log"
$DefaultURL = "https://" + $machineHostName + ":44368"

## This routine writes the output string to the console and also to a log file.
function Log-Info([string] $OutputText)
{
    $OutputText | %{ Write-Output $_; Out-File -filepath $logName -inputobject $_ -append -encoding "ASCII" }
}

## This routine writes the output string to the console and also to a log file.
function Log-Error([string] $OutputText)
{
    $OutputText | %{ Write-Error $_; Out-File -filepath $logName -inputobject $_ -append -encoding "ASCII" }
}

<#
.SYNOPSIS
    Adds Trusted Local domains to registry.
#>
function AddTrustedLocalDomains
{
    Log-Info "Adding trusted local domains."

    $machineName = (Get-WmiObject win32_computersystem).DNSHostName
    $domainName = (Get-WmiObject win32_computersystem).Domain
    $fqdn = $machineName+"."+$domainName
    
    Log-Info "machineName = $machineName"
    Log-Info "domainName = $domainName"
    Log-Info "fqdn = $fqdn"
    
    # List of trusted local domains.
    $trustedLocalDomains = New-Object System.Collections.Generic.List[string]
    $trustedLocalDomains.Add("localhost");
    $trustedLocalDomains.Add($machineName);
    $trustedLocalDomains.Add($fqdn);

    # Create trusted local domain registry keys.
    foreach ($domain in $trustedLocalDomains)
    {
        New-ItemProperty -Path $regKeyIEAllow -Name $domain -Value 0 -Force

        $regKeyEscDomain = $TrustedSitesEscDomainsRegistryHive + "\" + $domain
        $regKeyDomain = $TrustedSitesDomainsRegistryHive + "\" + $domain
        foreach ($regKey in $regKeyEscDomain,$regKeyDomain)
        {
            if ( -not (Test-Path $regKey))
            {
                Log-Info "Creating $regKey"
                New-Item -Path $regKey -Force
            }
            
            # Add https key and setting its value to 1.
            New-ItemProperty -Path $regKey -Name "https" -Value 1 -Force
        }
    }
}

<#
.SYNOPSIS
    Adds Trusted domains to registry.
#>
function AddTrustedDomains
{
    Log-Info "Adding trusted domains."
    
    # List of trusted domains.
    $trustedDomains = New-Object System.Collections.Generic.List[string]
    $trustedDomains.Add("*.live.com")
    $trustedDomains.Add("*.microsoft.com");
    $trustedDomains.Add("*.microsoftonline.com");
    $trustedDomains.Add("*.microsoftonline-p.com");
    $trustedDomains.Add("*.azure.net");
    $trustedDomains.Add("*.azure.com");
    $trustedDomains.Add("*.msecnd.net");
    $trustedDomains.Add("*.windows.net");
    $trustedDomains.Add("*.gfx.ms");
	$trustedDomains.Add("*.azuremigrate.blob.core.windows.net");
    $trustedDomains.Add("*.microsoft.ca1.qualtrics.com");
	$trustedDomains.Add("*.msftauth.net");
    $trustedDomains.Add("*.msauth.net");

    $AzureCoudKeyFullPath = $AzureMigrateApplianceHive + "\" + $AzureCloudKeyName;
    if (Test-Path $AzureCoudKeyFullPath)
    {
        $cloudName = (Get-ItemProperty $AzureMigrateApplianceHive -Name $AzureCloudKeyName).$AzureCloudKeyName
    }
    
    Log-Info "cloudName = $cloudName"
    
    if ($cloudName -eq "Mooncake")
    {
        $trustedDomains.Add("*.microsoftonline.cn");
        $trustedDomains.Add("*.microsoftonline-p.cn");
        $trustedDomains.Add("*.azure.cn");
    }
    elseif ($cloudName -eq "Blackforest")
    {
        $trustedDomains.Add("*.microsoftonline.de");
        $trustedDomains.Add("*.microsoftonline-p.de");
        $trustedDomains.Add("*.azure.de");
        $trustedDomains.Add("*.microsoftazure.de");
    }
    elseif ($cloudName -eq "Fairfax")
    {
        $trustedDomains.Add("*.microsoftonline.us");
        $trustedDomains.Add("*.microsoftonline-p.us");
        $trustedDomains.Add("*.azure.us");
    }
    
    # Create trusted domain registry keys.
    foreach ($domain in $trustedDomains)
    {
        $regKeyEscDomain = $TrustedSitesEscDomainsRegistryHive + "\" + $domain
        $regKeyDomain = $TrustedSitesDomainsRegistryHive + "\" + $domain
        
        foreach ($regKey in $regKeyEscDomain,$regKeyDomain)
        {
            if ( -not (Test-Path $regKey))
            {
                Log-Info "Creating $regKey"
                New-Item -Path $regKey -Force
            }
            
            # Add https key and setting its value to 2.
            New-ItemProperty -Path $regKey -Name "https" -Value 2 -Force
        }	
    }
}

<#
.SYNOPSIS
    Create blank registry keys.
#>
function AddBlankRegistryKeys
{
    foreach ($regKey in $regKeyEscBlankDomain,$regKeyBlankDomain)
    {
        if ( -not (Test-Path $regKey))
        {
            Log-Info "Creating $regKey"
            New-Item -Path $regKey -Force
        }
        
        # Add about registry key and setting its value to 2.
        New-ItemProperty -Path $regKey -Name "about" -Value 2 -Force
    }
}

###############
## Main flow ##
###############

try
{
    # List of registry hives.
    $registryHives = New-Object System.Collections.Generic.List[string]
    $registryHives.Add($InternetSettingsRegistryHive);
    $registryHives.Add($TrustedSitesEscDomainsRegistryHive);
    $registryHives.Add($TrustedSitesDomainsRegistryHive);
    $registryHives.Add($InternetExplorerRegistryHive);
    $registryHives.Add($regKeyIEAllow);

    # Create registry hives.
    foreach ($regHive in $registryHives)
    {
        if ( -not (Test-Path $regHive))
        {
            Log-Info "Creating $regHive"
            New-Item -Path $regHive -Force
        }
    }
     
    New-ItemProperty -Path $InternetExplorerRegistryHive -Name "PopupMgr" -Value 0 -Force
    New-ItemProperty -Path $InternetSettingsRegistryHive -Name "WarnonZoneCrossing" -Value 0 -Force

    AddTrustedLocalDomains
    AddTrustedDomains
    AddBlankRegistryKeys

    # Launch url.
    Log-Info "Launching url - $DefaultURL"
    Start $DefaultURL

    #Create a shortcut for launching appliance portal
    $Shell = New-Object -ComObject ("WScript.Shell")
    $ShortCut = $Shell.CreateShortcut($env:PUBLIC + "\Desktop\Azure Migrate Appliance Configuration Manager.lnk")
    $ShortCut.TargetPath = "C:\Program Files (x86)\Internet Explorer\iexplore.exe"
    $ShortCut.Arguments = $DefaultURL
    $ShortCut.WorkingDirectory = "C:\Program Files (x86)\Internet Explorer"
    $ShortCut.WindowStyle = 1
    $ShortCut.IconLocation = "C:\Program Files\Microsoft Azure Appliance Configuration Manager\favicon.ico"
    $ShortCut.Save()
}
catch
{
    Log-Error "Script execution failed with error: $_.Exception.Message"
    Log-Error "Error Record: $_.Exception.ErrorRecord"
    Log-Error "Exception caught:  $_.Exception"
    exit 1
}
# SIG # Begin signature block
# MIInLAYJKoZIhvcNAQcCoIInHTCCJxkCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCC690wHdRA5x2Yf
# Jyc7O7CxkSR5xXE9BcWIfy09gbC1/qCCEWUwggh3MIIHX6ADAgECAhM2AAAA4kR3
# G3EijXHTAAEAAADiMA0GCSqGSIb3DQEBCwUAMEExEzARBgoJkiaJk/IsZAEZFgNH
# QkwxEzARBgoJkiaJk/IsZAEZFgNBTUUxFTATBgNVBAMTDEFNRSBDUyBDQSAwMTAe
# Fw0xOTAzMjUxMzEzMzRaFw0yMDAzMjQxMzEzMzRaMCQxIjAgBgNVBAMTGU1pY3Jv
# c29mdCBBenVyZSBDb2RlIFNpZ24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEK
# AoIBAQCq3seGPV6z/qGZ5OKvkCZdIhfURhgoq4H4VtLGQ32NUqeRsbIQAa5cjUvu
# hbgtJlj3z2trVo7R9N1Qxega6uWrUZj8Zh7wTvevxb65aLLpr12F/w1LFhy3J7LX
# c0WQQD0uLB9AY2Gln88LuQyC8CvE40lrjm055fxnQmLP4dAXRK0KZA4ml4aHEOQl
# 9hd5Bh0sxn6mHCNMmYR3su+SwebaXWfs/Rluv+lC6Uwu9DgCktYU+O2xhitCupSS
# vSoYwgO5MLW6IfQg5WtA86e+3jFMOCQoZYDk1m6aYi2zuwwzYuk2BdAej5sVvvz5
# ZLDYjvAQy3zndQT10EdkTesjYg0ZAgMBAAGjggWDMIIFfzApBgkrBgEEAYI3FQoE
# HDAaMAwGCisGAQQBgjdbAgEwCgYIKwYBBQUHAwMwPQYJKwYBBAGCNxUHBDAwLgYm
# KwYBBAGCNxUIhpDjDYTVtHiE8Ys+hZvdFs6dEoFghNiuSIWmliYCAWQCAQ0wggJ2
# BggrBgEFBQcBAQSCAmgwggJkMGIGCCsGAQUFBzAChlZodHRwOi8vY3JsLm1pY3Jv
# c29mdC5jb20vcGtpaW5mcmEvQ2VydHMvQlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1F
# JTIwQ1MlMjBDQSUyMDAxKDEpLmNydDBSBggrBgEFBQcwAoZGaHR0cDovL2NybDEu
# YW1lLmdibC9haWEvQlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1FJTIwQ1MlMjBDQSUy
# MDAxKDEpLmNydDBSBggrBgEFBQcwAoZGaHR0cDovL2NybDIuYW1lLmdibC9haWEv
# QlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1FJTIwQ1MlMjBDQSUyMDAxKDEpLmNydDBS
# BggrBgEFBQcwAoZGaHR0cDovL2NybDMuYW1lLmdibC9haWEvQlkyUEtJQ1NDQTAx
# LkFNRS5HQkxfQU1FJTIwQ1MlMjBDQSUyMDAxKDEpLmNydDBSBggrBgEFBQcwAoZG
# aHR0cDovL2NybDQuYW1lLmdibC9haWEvQlkyUEtJQ1NDQTAxLkFNRS5HQkxfQU1F
# JTIwQ1MlMjBDQSUyMDAxKDEpLmNydDCBrQYIKwYBBQUHMAKGgaBsZGFwOi8vL0NO
# PUFNRSUyMENTJTIwQ0ElMjAwMSxDTj1BSUEsQ049UHVibGljJTIwS2V5JTIwU2Vy
# dmljZXMsQ049U2VydmljZXMsQ049Q29uZmlndXJhdGlvbixEQz1BTUUsREM9R0JM
# P2NBQ2VydGlmaWNhdGU/YmFzZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0
# aG9yaXR5MB0GA1UdDgQWBBRS47CtDNi0cKTZcPAA30itV+wMEjAOBgNVHQ8BAf8E
# BAMCB4AwUAYDVR0RBEkwR6RFMEMxKTAnBgNVBAsTIE1pY3Jvc29mdCBPcGVyYXRp
# b25zIFB1ZXJ0byBSaWNvMRYwFAYDVQQFEw0yMzYxNjcrNDUzMzgwMIIB1AYDVR0f
# BIIByzCCAccwggHDoIIBv6CCAbuGPGh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9w
# a2lpbmZyYS9DUkwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDEu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDIu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDMu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIYuaHR0cDovL2NybDQu
# YW1lLmdibC9jcmwvQU1FJTIwQ1MlMjBDQSUyMDAxLmNybIaBumxkYXA6Ly8vQ049
# QU1FJTIwQ1MlMjBDQSUyMDAxLENOPUJZMlBLSUNTQ0EwMSxDTj1DRFAsQ049UHVi
# bGljJTIwS2V5JTIwU2VydmljZXMsQ049U2VydmljZXMsQ049Q29uZmlndXJhdGlv
# bixEQz1BTUUsREM9R0JMP2NlcnRpZmljYXRlUmV2b2NhdGlvbkxpc3Q/YmFzZT9v
# YmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Qb2ludDAfBgNVHSMEGDAWgBQbZqIZ
# /JvrpdqEjxiY6RCkw3uSvTAfBgNVHSUEGDAWBgorBgEEAYI3WwIBBggrBgEFBQcD
# AzANBgkqhkiG9w0BAQsFAAOCAQEAaYwZUZEKV4ZrTtIa0G/BCXxv9n+2jS3Tgnxb
# BlSPriM5WvURcRv83qyIUeswlVbBnbkOsGME5Q/+8w6shIYz03jNeC+3Sq4dO3rC
# vrMalQO35SRX55Xm+LJZS9N/y8/YDs7zuI1zC4g80oBtwfCZ6faeYYX1afCX2LXO
# ekAtLmJzI+n7KFhdo/38JQKF8km4n+A6ELzf+7rwIaJzecHtBmUlZkgWr+rl5g7G
# fbgUUDnNCC0BOKhEdTL6OYCLT1YvmpoqDg+mX4vg52c8ZMS9HVaObacYqKTE6cdR
# pb5fJKKSE0Ybe+PrPp4TwSvfJ2YO/ElKPEMKfZRgYFKRUhpF0jCCCOYwggbOoAMC
# AQICEx8AAAAUtMUfxvKAvnEAAAAAABQwDQYJKoZIhvcNAQELBQAwPDETMBEGCgmS
# JomT8ixkARkWA0dCTDETMBEGCgmSJomT8ixkARkWA0FNRTEQMA4GA1UEAxMHYW1l
# cm9vdDAeFw0xNjA5MTUyMTMzMDNaFw0yMTA5MTUyMTQzMDNaMEExEzARBgoJkiaJ
# k/IsZAEZFgNHQkwxEzARBgoJkiaJk/IsZAEZFgNBTUUxFTATBgNVBAMTDEFNRSBD
# UyBDQSAwMTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBANVXgQLW+frQ
# 9xuAud03zSTcZmH84YlyrSkM0hsbmr+utG00tVRHgw40pxYbJp5W+hpDwnmJgicF
# oGRrPt6FifMmnd//1aD/fW1xvGs80yZk9jxTNcisVF1CYIuyPctwuJZfwE3wcGxh
# kVw/tj3ZHZVacSls3jRD1cGwrcVo1IR6+hHMvUejtt4/tv0UmUoH82HLQ8w1oTX9
# D7xj35Zt9T0pOPqM3Gt9+/zs7tPp2gyoOYv8xR4X0iWZKuXTzxugvMA63YsB4ehu
# SBqzHdkF55rxH47aT6hPhvDHlm7M2lsZcRI0CUAujwcJ/vELeFapXNGpt2d3wcPJ
# M0bpzrPDJ/8CAwEAAaOCBNowggTWMBAGCSsGAQQBgjcVAQQDAgEBMCMGCSsGAQQB
# gjcVAgQWBBSR/DPOQp72k+bifVTXCBi7uNdxZTAdBgNVHQ4EFgQUG2aiGfyb66Xa
# hI8YmOkQpMN7kr0wggEEBgNVHSUEgfwwgfkGBysGAQUCAwUGCCsGAQUFBwMBBggr
# BgEFBQcDAgYKKwYBBAGCNxQCAQYJKwYBBAGCNxUGBgorBgEEAYI3CgMMBgkrBgEE
# AYI3FQYGCCsGAQUFBwMJBggrBgEFBQgCAgYKKwYBBAGCN0ABAQYLKwYBBAGCNwoD
# BAEGCisGAQQBgjcKAwQGCSsGAQQBgjcVBQYKKwYBBAGCNxQCAgYKKwYBBAGCNxQC
# AwYIKwYBBQUHAwMGCisGAQQBgjdbAQEGCisGAQQBgjdbAgEGCisGAQQBgjdbAwEG
# CisGAQQBgjdbBQEGCisGAQQBgjdbBAEGCisGAQQBgjdbBAIwGQYJKwYBBAGCNxQC
# BAweCgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMBIGA1UdEwEB/wQIMAYBAf8CAQAw
# HwYDVR0jBBgwFoAUKV5RXmSuNLnrrJwNp4x1AdEJCygwggFoBgNVHR8EggFfMIIB
# WzCCAVegggFToIIBT4YjaHR0cDovL2NybDEuYW1lLmdibC9jcmwvYW1lcm9vdC5j
# cmyGMWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2lpbmZyYS9jcmwvYW1lcm9v
# dC5jcmyGI2h0dHA6Ly9jcmwyLmFtZS5nYmwvY3JsL2FtZXJvb3QuY3JshiNodHRw
# Oi8vY3JsMy5hbWUuZ2JsL2NybC9hbWVyb290LmNybIaBqmxkYXA6Ly8vQ049YW1l
# cm9vdCxDTj1BTUVST09ULENOPUNEUCxDTj1QdWJsaWMlMjBLZXklMjBTZXJ2aWNl
# cyxDTj1TZXJ2aWNlcyxDTj1Db25maWd1cmF0aW9uLERDPUFNRSxEQz1HQkw/Y2Vy
# dGlmaWNhdGVSZXZvY2F0aW9uTGlzdD9iYXNlP29iamVjdENsYXNzPWNSTERpc3Ry
# aWJ1dGlvblBvaW50MIIBqwYIKwYBBQUHAQEEggGdMIIBmTA3BggrBgEFBQcwAoYr
# aHR0cDovL2NybDEuYW1lLmdibC9haWEvQU1FUk9PVF9hbWVyb290LmNydDBHBggr
# BgEFBQcwAoY7aHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraWluZnJhL2NlcnRz
# L0FNRVJPT1RfYW1lcm9vdC5jcnQwNwYIKwYBBQUHMAKGK2h0dHA6Ly9jcmwyLmFt
# ZS5nYmwvYWlhL0FNRVJPT1RfYW1lcm9vdC5jcnQwNwYIKwYBBQUHMAKGK2h0dHA6
# Ly9jcmwzLmFtZS5nYmwvYWlhL0FNRVJPT1RfYW1lcm9vdC5jcnQwgaIGCCsGAQUF
# BzAChoGVbGRhcDovLy9DTj1hbWVyb290LENOPUFJQSxDTj1QdWJsaWMlMjBLZXkl
# MjBTZXJ2aWNlcyxDTj1TZXJ2aWNlcyxDTj1Db25maWd1cmF0aW9uLERDPUFNRSxE
# Qz1HQkw/Y0FDZXJ0aWZpY2F0ZT9iYXNlP29iamVjdENsYXNzPWNlcnRpZmljYXRp
# b25BdXRob3JpdHkwDQYJKoZIhvcNAQELBQADggIBACi3Soaajx+kAWjNwgDqkIvK
# AOFkHmS1t0DlzZlpu1ANNfA0BGtck6hEG7g+TpUdVrvxdvPQ5lzU3bGTOBkyhGmX
# oSIlWjKC7xCbbuYegk8n1qj3rTcjiakdbBqqHdF8J+fxv83E2EsZ+StzfCnZXA62
# QCMn6t8mhCWBxpwPXif39Ua32yYHqP0QISAnLTjjcH6bAV3IIk7k5pQ/5NA6qIL8
# yYD6vRjpCMl/3cZOyJD81/5+POLNMx0eCClOfFNxtaD0kJmeThwL4B2hAEpHTeRN
# tB8ib+cze3bvkGNPHyPlSHIuqWoC31x2Gk192SfzFDPV1PqFOcuKjC8049SSBtC1
# X7hyvMqAe4dop8k3u25+odhvDcWdNmimdMWvp/yZ6FyjbGlTxtUqE7iLTLF1eaUL
# SEobAap16hY2N2yTJTISKHzHI4rjsEQlvqa2fj6GLxNj/jC+4LNy+uRmfQXShd30
# lt075qTroz0Nt680pXvVhsRSdNnzW2hfQu2xuOLg8zKGVOD/rr0GgeyhODjKgL2G
# Hxctbb9XaVSDf6ocdB//aDYjiabmWd/WYmy7fQ127KuasMh5nSV2orMcAed8CbIV
# I3NYu+sahT1DRm/BGUN2hSpdsPQeO73wYvp1N7DdLaZyz7XsOCx1quCwQ+bojWVQ
# TmKLGegSoUpZNfmP9MtSMYIVHTCCFRkCAQEwWDBBMRMwEQYKCZImiZPyLGQBGRYD
# R0JMMRMwEQYKCZImiZPyLGQBGRYDQU1FMRUwEwYDVQQDEwxBTUUgQ1MgQ0EgMDEC
# EzYAAADiRHcbcSKNcdMAAQAAAOIwDQYJYIZIAWUDBAIBBQCgga4wGQYJKoZIhvcN
# AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUw
# LwYJKoZIhvcNAQkEMSIEIDgBS7ZN1WJGWT3GsLm6JiMt1+9z/WUr8zap4xrrTnpT
# MEIGCisGAQQBgjcCAQwxNDAyoBSAEgBNAGkAYwByAG8AcwBvAGYAdKEagBhodHRw
# Oi8vd3d3Lm1pY3Jvc29mdC5jb20wDQYJKoZIhvcNAQEBBQAEggEAByZX3QPN1Mro
# SWZyj9Iwrev0Du83v/6gCb59q0pVlr/eUDsFmvDBZ8UgOuqdXM8MuVM0HNxR/NDt
# 8KRWCzJD1gebxhOXyAPc8XlIfpOapOS6J25o7Q3f2jtEPXUDi7+p4947RSKIyp0Z
# b/vbfSPw87vRSDbbC2FqxYy4q+fQPDY5lhhLUnCadx4+ldLjHbpY+7AcXnOymgFZ
# 6WKuxcwB9EsXPjZfPfRPssWPrgF2jkBeib1vHieFnilPXIE668a2Ffm8y3vzZ4Su
# 7q8bUZCbFLZycPDfr3RLoT6e98ReUsaNDO1+GnYFCudVs8tFoI0uqwiUiY1Kkvpc
# 0dZxfpROc6GCEuUwghLhBgorBgEEAYI3AwMBMYIS0TCCEs0GCSqGSIb3DQEHAqCC
# Er4wghK6AgEDMQ8wDQYJYIZIAWUDBAIBBQAwggFRBgsqhkiG9w0BCRABBKCCAUAE
# ggE8MIIBOAIBAQYKKwYBBAGEWQoDATAxMA0GCWCGSAFlAwQCAQUABCB9L7r+5qex
# t8PbKr8VE3xJjpW5jcCQhxsLlQ0JvdQz7AIGXioHhGlKGBMyMDIwMDEyNjE5MDk1
# MC4wNjRaMASAAgH0oIHQpIHNMIHKMQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2Fz
# aGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENv
# cnBvcmF0aW9uMSUwIwYDVQQLExxNaWNyb3NvZnQgQW1lcmljYSBPcGVyYXRpb25z
# MSYwJAYDVQQLEx1UaGFsZXMgVFNTIEVTTjpBRTJDLUUzMkItMUFGQzElMCMGA1UE
# AxMcTWljcm9zb2Z0IFRpbWUtU3RhbXAgU2VydmljZaCCDjwwggTxMIID2aADAgEC
# AhMzAAABFpMi6r+7LU3mAAAAAAEWMA0GCSqGSIb3DQEBCwUAMHwxCzAJBgNVBAYT
# AlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYD
# VQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBU
# aW1lLVN0YW1wIFBDQSAyMDEwMB4XDTE5MTExMzIxNDAzNFoXDTIxMDIxMTIxNDAz
# NFowgcoxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQH
# EwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJTAjBgNV
# BAsTHE1pY3Jvc29mdCBBbWVyaWNhIE9wZXJhdGlvbnMxJjAkBgNVBAsTHVRoYWxl
# cyBUU1MgRVNOOkFFMkMtRTMyQi0xQUZDMSUwIwYDVQQDExxNaWNyb3NvZnQgVGlt
# ZS1TdGFtcCBTZXJ2aWNlMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# 0Pgb8/296ie/Lj2rWq+MIlMZwkSUwZsIKd472tyeVOyNcKgqSCT4zQvz2kd+VD7l
# YWN3V0USL5oipdp+xp7wH7CAHC7zNU21PjdHWPOi2okIlPyTikrQBowo+MOV9Xgd
# 3WqMnJSKEank7QmSHgJimJ2q/ZRR5+0Z5uZRejJHkQcJmTB8Gq/wg2E/gjuRl/iG
# a4fGJu0cHSUiX78m5FEyaac1XnkqafSqYR8qb7sn3ZVt/ltbiGUJr874oi2bZduU
# tCMR0QiWWfBMExcLV4A6ermC98cbbvi/pQb1p1l7vXT2NReD+xkFqzKn0cA3Vi9c
# c5LjDhY91L18RuHIgU3qHQIDAQABo4IBGzCCARcwHQYDVR0OBBYEFOW/Xiu4F+gX
# zUflH3k0/lfIIVULMB8GA1UdIwQYMBaAFNVjOlyKMZDzQ3t8RhvFM2hahW1VMFYG
# A1UdHwRPME0wS6BJoEeGRWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
# L3Byb2R1Y3RzL01pY1RpbVN0YVBDQV8yMDEwLTA3LTAxLmNybDBaBggrBgEFBQcB
# AQROMEwwSgYIKwYBBQUHMAKGPmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kv
# Y2VydHMvTWljVGltU3RhUENBXzIwMTAtMDctMDEuY3J0MAwGA1UdEwEB/wQCMAAw
# EwYDVR0lBAwwCgYIKwYBBQUHAwgwDQYJKoZIhvcNAQELBQADggEBADaDatfaqaPb
# Ay/pSdK8e8XdzN6v9979NSWLUsNHoNBFpyr1FTGcvwf0SKIfe0ygt8s8plkAYxMU
# ftUmOnO+OnGXUgTOreXIw4ztsepotreHcL094+bn7OUGLPMa56GQii3WUgiGPP0g
# fNXhXcqSdd9HmXjMhKfRn0jOKREJTPqPHLXSxcA1SVTrg8JDtkD+yWVzuuAkSopT
# GxtJp5PcrYUrMb7nW1coIe7tsQiSPp6xFVzKfXFUJ9VzAChucE+8pqXLpV/xU3p/
# 1vf0DgLZMpI22mwAgbe/E6wgyDSKyHXI4UsiIlSYASv+IlKOtcXzrXV0IRQUdRyI
# C1ZiWWL/YggwggZxMIIEWaADAgECAgphCYEqAAAAAAACMA0GCSqGSIb3DQEBCwUA
# MIGIMQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
# UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMTIwMAYDVQQD
# EylNaWNyb3NvZnQgUm9vdCBDZXJ0aWZpY2F0ZSBBdXRob3JpdHkgMjAxMDAeFw0x
# MDA3MDEyMTM2NTVaFw0yNTA3MDEyMTQ2NTVaMHwxCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1w
# IFBDQSAyMDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqR0NvHcR
# ijog7PwTl/X6f2mUa3RUENWlCgCChfvtfGhLLF/Fw+Vhwna3PmYrW/AVUycEMR9B
# GxqVHc4JE458YTBZsTBED/FgiIRUQwzXTbg4CLNC3ZOs1nMwVyaCo0UN0Or1R4HN
# vyRgMlhgRvJYR4YyhB50YWeRX4FUsc+TTJLBxKZd0WETbijGGvmGgLvfYfxGwScd
# JGcSchohiq9LZIlQYrFd/XcfPfBXday9ikJNQFHRD5wGPmd/9WbAA5ZEfu/QS/1u
# 5ZrKsajyeioKMfDaTgaRtogINeh4HLDpmc085y9Euqf03GS9pAHBIAmTeM38vMDJ
# RF1eFpwBBU8iTQIDAQABo4IB5jCCAeIwEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
# BBYEFNVjOlyKMZDzQ3t8RhvFM2hahW1VMBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIA
# QwBBMAsGA1UdDwQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2
# VsuP6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeGRWh0dHA6Ly9jcmwu
# bWljcm9zb2Z0LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEw
# LTA2LTIzLmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0dHA6Ly93
# d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYt
# MjMuY3J0MIGgBgNVHSABAf8EgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9BggrBgEF
# BQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL1BLSS9kb2NzL0NQUy9kZWZh
# dWx0Lmh0bTBABggrBgEFBQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
# eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG9w0BAQsFAAOCAgEAB+aI
# UQ3ixuCYP4FxAz2do6Ehb7Prpsz1Mb7PBeKp/vpXbRkws8LFZslq3/Xn8Hi9x6ie
# JeP5vO1rVFcIK1GCRBL7uVOMzPRgEop2zEBAQZvcXBf/XPleFzWYJFZLdO9CEMiv
# v3/Gf/I3fVo/HPKZeUqRUgCvOA8X9S95gWXZqbVr5MfO9sp6AG9LMEQkIjzP7QOl
# lo9ZKby2/QThcJ8ySif9Va8v/rbljjO7Yl+a21dA6fHOmWaQjP9qYn/dxUoLkSbi
# OewZSnFjnXshbcOco6I8+n99lmqQeKZt0uGc+R38ONiU9MalCpaGpL2eGq4EQoO4
# tYCbIjggtSXlZOz39L9+Y1klD3ouOVd2onGqBooPiRa6YacRy5rYDkeagMXQzafQ
# 732D8OE7cQnfXXSYIghh2rBQHm+98eEA3+cxB6STOvdlR3jo+KhIq/fecn5ha293
# qYHLpwmsObvsxsvYgrRyzR30uIUBHoD7G4kqVDmyW9rIDVWZeodzOwjmmC3qjeAz
# LhIp9cAvVCch98isTtoouLGp25ayp0Kiyc8ZQU3ghvkqmqMRZjDTu3QyS99je/WZ
# ii8bxyGvWbWu3EQ8l1Bx16HSxVXjad5XwdHeMMD9zOZN+w2/XU/pnR4ZOC+8z1gF
# Lu8NoFA12u8JJxzVs341Hgi62jbb01+P3nSISRKhggLOMIICNwIBATCB+KGB0KSB
# zTCByjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcT
# B1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjElMCMGA1UE
# CxMcTWljcm9zb2Z0IEFtZXJpY2EgT3BlcmF0aW9uczEmMCQGA1UECxMdVGhhbGVz
# IFRTUyBFU046QUUyQy1FMzJCLTFBRkMxJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1l
# LVN0YW1wIFNlcnZpY2WiIwoBATAHBgUrDgMCGgMVAIdNW9zyT6CLG1qCDNc++szs
# 3ZZDoIGDMIGApH4wfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
# EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
# bjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwDQYJKoZI
# hvcNAQEFBQACBQDh19HAMCIYDzIwMjAwMTI2MTY1MjE2WhgPMjAyMDAxMjcxNjUy
# MTZaMHcwPQYKKwYBBAGEWQoEATEvMC0wCgIFAOHX0cACAQAwCgIBAAICHGMCAf8w
# BwIBAAICEgYwCgIFAOHZI0ACAQAwNgYKKwYBBAGEWQoEAjEoMCYwDAYKKwYBBAGE
# WQoDAqAKMAgCAQACAwehIKEKMAgCAQACAwGGoDANBgkqhkiG9w0BAQUFAAOBgQBE
# wpav9jy3+4EkbfKL+Sgxd4R4OKrQ36sx1HRqogHHuz6a2hXoXRx92HCnwsQCjjA0
# Q9LYBRThcJHGubdjNfoFiLSGpQC3lrW7QxmJveB9+rzG7NQwB9Vdf4yS+IzYnyo8
# t8RUUFrm6LVLhihwUfI5GUfigBWGsIxnwVjl8a4z7zGCAw0wggMJAgEBMIGTMHwx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRt
# b25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJjAkBgNVBAMTHU1p
# Y3Jvc29mdCBUaW1lLVN0YW1wIFBDQSAyMDEwAhMzAAABFpMi6r+7LU3mAAAAAAEW
# MA0GCWCGSAFlAwQCAQUAoIIBSjAaBgkqhkiG9w0BCQMxDQYLKoZIhvcNAQkQAQQw
# LwYJKoZIhvcNAQkEMSIEICL6stAAF06qmPp9LiJNqRkEkfuppP4FVU3yxL708MSm
# MIH6BgsqhkiG9w0BCRACLzGB6jCB5zCB5DCBvQQggyKU9qRgKQiXXCmbITbdtLEN
# hYxqIMhBaM+iXtLBkMowgZgwgYCkfjB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
# V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
# IENvcnBvcmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0Eg
# MjAxMAITMwAAARaTIuq/uy1N5gAAAAABFjAiBCA46A1aC37L49Ix2k6KLXDVDa/r
# J9NPoiQM2R0cYrsGkzANBgkqhkiG9w0BAQsFAASCAQAFYTIf7yEM0/TSxpU3O0jq
# atAfffiJ2xN6E75guMGG+XMEvF6LnFYBda8KUNi2xrRc0srf+5eOfpUu65hFoExO
# Ax5FTMatXeDoFkB6YKVLciDOMIxW29Rhnuh3Xuc6S0sZPGpw3jcMb1x3atylbWRT
# 8NSm4FWVybuC6e3EyCafN+/k5IDJeAbmm+O3OJsrJqCvsuDO/DudYCo5pqWkdgw1
# XZS9WsGZOchRrwjjFF6gIdxs4IHNGrpJHO6OrD/+vG3lrhCXZx8mQsB+xKEBaB7M
# HMiWDSDzi/Rf3nOQ+eNgDFL1CtsfhXQbYh6ZKBbhl39A0/BpoVUsqHx8qtojm75u
# SIG # End signature block
